package be.iminds.ilabt.jfed.experimenter_gui.ui;

import be.iminds.ilabt.jfed.experimenter_gui.config.JFedGuiConfig;
import be.iminds.ilabt.jfed.experimenter_gui.util.BrowserUtil;
import be.iminds.ilabt.jfed.experimenter_gui.util.DateTimeUtils;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.*;
import be.iminds.ilabt.jfed.ui.javafx.FXMLUtil;
import be.iminds.ilabt.jfed.util.TimeUtil;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.beans.value.ChangeListener;
import javafx.collections.ListChangeListener;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.geometry.Insets;
import javafx.scene.control.*;
import javafx.scene.layout.*;
import javafx.scene.paint.Color;
import org.controlsfx.control.PopOver;
import org.controlsfx.glyphfont.Glyph;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.inject.Inject;
import javax.validation.constraints.Null;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.*;

/**
 * User: twalcari
 * Date: 9/16/2015
 * Time: 14:52
 */
public class HealthStatusPane extends StackPane {
    private static final Logger LOG = LoggerFactory.getLogger(HealthStatusPane.class);

    private static final Comparator<? super MaintenanceInfo> MAINTENANCE_MESSAGE_COMPARATOR =
            Comparator.nullsLast(Comparator.comparing(MaintenanceInfo::getStart))
                    .thenComparing(Comparator.nullsLast(Comparator.comparing(MaintenanceInfo::getEnd)))
                    .thenComparing(Comparator.nullsLast(Comparator.comparing(MaintenanceInfo::getPlanned)))
                    .thenComparing(MaintenanceInfo::getMessage)
                    .thenComparing(Comparator.nullsLast(Comparator.comparing(MaintenanceInfo::getReported)));

    @FXML
    private Label noInfoLabel;
    @FXML
    private GridPane infoGridPane;

    @FXML
    private Label healthKeyLabel;
    @FXML
    private Label healthLabel;
    @FXML
    private Label reputationLabel;
    @FXML
    private Label freeLabel;
    @FXML
    private Label freeInfoLabel;
    @FXML
    private Label freeHwInfoLabel;
    @FXML
    private Label ipFreeLabel;
    @FXML
    private Label ipFreeInfoLabel;
    @FXML
    private Label maintenanceLabel;

    @FXML
    private Glyph healthGlyph;
    @FXML
    private Glyph reputationGlyph;
    @FXML
    private AvailabilityMeter availabiltyMeter;
    @FXML
    private AvailabilityMeter ipAvailabiltyMeter;
    @FXML
    private Glyph maintenanceGlyph;

    @FXML
    private Button hwFreeInfoButton;

    @FXML
    private Button maintenanceInfoButton;

    @FXML
    private Label updateHealthDateLabel;
    @FXML
    private Label updateCountsDateLabel;
    @FXML
    private Button refreshButton;
    @FXML
    private Glyph refreshGlyph;

    @FXML
    private Hyperlink testbedInfoButton;
    @FXML
    private Hyperlink testbedDocumentationButton;

//    private boolean showHwFree = false;

    private JFedGuiConfig config;
    private Server server;

    private final ObjectProperty<JFedGuiConfig.ResourceType> type = new SimpleObjectProperty<>(null);
    private final StringProperty documentationLink = new SimpleStringProperty();
    private final StringProperty infoLink = new SimpleStringProperty();
//    private final ObjectProperty<ServerGlimpse> serverGlimpse = new SimpleOb;jectProperty<>(null);

    private BrowserUtil browserUtil;

    @Inject
    public HealthStatusPane() {
        FXMLUtil.injectFXML(this);

        availabiltyMeter.useVmCalculationMethodProperty().bind(type.isEqualTo(JFedGuiConfig.ResourceType.VM));
        testbedInfoButton.disableProperty().bind(infoLink.isEmpty());
        testbedInfoButton.setOnAction(event -> HealthStatusPane.this.browserUtil.openUrlInBrowser(infoLink.getValue()));

        testbedDocumentationButton.disableProperty().bind(documentationLink.isEmpty());
        testbedDocumentationButton.setOnAction(event -> HealthStatusPane.this.browserUtil.openUrlInBrowser(documentationLink.getValue()));
    }

    public void setBrowserUtil(BrowserUtil browserUtil) {
        this.browserUtil = browserUtil;
    }

    @FXML
    public void initialize() {
    }

    public void setConfig(@Nonnull JFedGuiConfig config) {
        reputationGlyph.setVisible(config.isShowReputationEnabled());
        this.config = config;

        if (server != null)
            setServer(server);
    }

    private Runnable unregisterServer;

    public void setServer(Server server) {
        this.server = server;
        if (unregisterServer != null)
            unregisterServer.run();

        if (config == null)
            return;

        boolean noInfo = true;
        if (config.getServerNodeCounts(server).getValue() != null &&
                config.getServerNodeCounts(server).getValue().getCountEntriesSize() > 0)
            noInfo = false;
        if (noInfo && !config.getServerHealthInfo(server).isBound())
            noInfo = false;
        if (noInfo && !config.getServerMaintenance(server).isEmpty())
            noInfo = false;
        if (noInfo && !config.getServerReputation(server).isBound())
            noInfo = false;
        if (noInfo) {
            noInfoLabel.setVisible(true);
            infoGridPane.setVisible(false);
            return;
        }
        noInfoLabel.setVisible(false);
        infoGridPane.setVisible(true);

        ChangeListener<ServerGlimpse.HealthInfo> healthListener =
                (observable, oldValue, newValue) -> showHealthInfo(newValue);
        config.getServerHealthInfo(server).addListener(healthListener);
        showHealthInfo(config.getServerHealthInfo(server).get());

        ChangeListener<Integer> reputationListener =
                (observable, oldValue, newValue) -> showReputation(newValue);
        config.getServerReputation(server).addListener(reputationListener);
        showReputation(config.getServerReputation(server).get());

        showLinks(server);

        ListChangeListener<? super MaintenanceInfo> maintenanceChangeListener =
                (ListChangeListener.Change<? extends MaintenanceInfo> change) ->
                        showMaintenance(change.getList());
        config.getServerMaintenance(server).addListener(maintenanceChangeListener);
        showMaintenance(config.getServerMaintenance(server));

        ChangeListener<AdvertisementCounts> serverNodeCountChangeListener =
                (observable, oldValue, newValue) -> showCounts(newValue);
        config.getServerNodeCounts(server).addListener(serverNodeCountChangeListener);
        showCounts(config.getServerNodeCounts(server).get());

        unregisterServer = () -> {
            config.getServerHealthInfo(server).removeListener(healthListener);
            config.getServerReputation(server).removeListener(reputationListener);
            config.getServerMaintenance(server).removeListener(maintenanceChangeListener);
            config.getServerNodeCounts(server).removeListener(serverNodeCountChangeListener);
            showLinks(null);
        };
    }

    public void showHealthInfo(@Nullable ServerGlimpse.HealthInfo healthInfo) {
        if (healthInfo == null) {
            healthLabel.setText("No Health info");
            return;
        }

        Tooltip healthTooltip;
        if (healthInfo.getHealthExplanationFull() != null)
            healthTooltip = new Tooltip(healthInfo.getHealthExplanationFull());
        else
            healthTooltip = new Tooltip("Health explanation unknown");

        healthLabel.setText(healthInfo.getHealth() != null ? healthInfo.getHealth() + "%" : "-");
        setGlyphColor(healthGlyph, healthInfo.getHealth());

        healthLabel.setTooltip(healthTooltip);
        healthKeyLabel.setTooltip(healthTooltip);
        healthGlyph.setTooltip(healthTooltip);

        String healthTime;
        if (healthInfo.getTimestamp() != null) {
            healthTime = TimeUtil.calcAgo(healthInfo.getTimestamp());
        } else {
            healthTime = "at unknown time";
        }

        updateHealthDateLabel.setText("Health updated " + healthTime + ".");
    }

    public void showReputation(@Nullable Integer reputation) {
        reputationLabel.setText(reputation != null ? reputation + "%" : "-");
        setGlyphColor(reputationGlyph, reputation);
    }

    public void showLinks(@Nullable Server server) {
        Testbed testbed = server == null ? null : server.getTestbed();

        infoLink.setValue(testbed == null ? null : testbed.getInfoUrl());
        documentationLink.setValue(testbed == null ? null : testbed.getHelpUrl());
    }

    public void showCounts(@Nullable AdvertisementCounts counts) {
        LOG.debug("HealthStatusPane count update: counts==null -> {}  type.get()={}", counts == null, type.get());
        if (counts != null) {
            if (type.get() != null) {
                Integer countAvail;
                Integer countTotal;
                switch (type.get()) {
                    case RAW_PC: {
                        countAvail = counts.getCount(ServerGlimpse.AdvertisementInfo.CountedObject.RAW_PC, ServerGlimpse.AdvertisementInfo.CountScope.AVAILABLE);
                        countTotal = counts.getCount(ServerGlimpse.AdvertisementInfo.CountedObject.RAW_PC, ServerGlimpse.AdvertisementInfo.CountScope.ALL);
                        break;
                    }
                    case VM: {
                        countAvail = counts.getCount(ServerGlimpse.AdvertisementInfo.CountedObject.VM, ServerGlimpse.AdvertisementInfo.CountScope.AVAILABLE);
                        countTotal = counts.getCount(ServerGlimpse.AdvertisementInfo.CountedObject.VM, ServerGlimpse.AdvertisementInfo.CountScope.ALL);
                        break;
                    }
                    default: {
                        countAvail = counts.getCount(ServerGlimpse.AdvertisementInfo.CountedObject.ANY, ServerGlimpse.AdvertisementInfo.CountScope.AVAILABLE);
                        countTotal = counts.getCount(ServerGlimpse.AdvertisementInfo.CountedObject.ANY, ServerGlimpse.AdvertisementInfo.CountScope.ALL);
                        break;
                    }
                }
                LOG.debug("HealthStatusPane count update: {}/{}", countAvail, countTotal);
                if (countAvail != null && countTotal != null && countTotal != 0) {
                    freeLabel.setText("Free " + type.get().getFriendlyName() + ":");

                    availabiltyMeter.setAvailable(countAvail);
                    availabiltyMeter.setTotal(countTotal);
                    availabiltyMeter.setUndetermined(false);

                    freeInfoLabel.setText(countAvail + " of " + countTotal);
//                    if (!showHwFree) {
//                        freeHwInfoLabel.setVisible(false);
////                        freeInfoLabel.setText(countAvail + " of " + countTotal);
//                    } else {
//                        freeHwInfoLabel.setVisible(true);
//                        LOG.debug("health is showing hw type counts: "+counts.getHwCountKeys());
////                        String freeText = countAvail + " of " + countTotal + "\n";
//                        String freeText = "";
//                        for (String hwType : counts.getHwCountKeys()) {
//                            Integer allCount = counts.getHwCount(hwType, AdvertisementCounts.CountScope.ALL);
//                            Integer availCount = counts.getHwCount(hwType, AdvertisementCounts.CountScope.AVAILABLE);
//                            freeText += hwType + ": " + availCount + " of " + allCount + "\n";
//                        }
////                        freeInfoLabel.setText(freeText);
//                        freeHwInfoLabel.setText(freeText);
//                    }
                    hwFreeInfoButton.setVisible(!counts.getHwCountKeys().isEmpty());
                    hwFreeInfoButton.setManaged(!counts.getHwCountKeys().isEmpty());
                } else {
                    freeLabel.setText("Free resources:");
                    availabiltyMeter.setAvailable(0);
                    availabiltyMeter.setTotal(0);
                    availabiltyMeter.setUndetermined(true);
                    freeInfoLabel.setText("-");
                    freeHwInfoLabel.setVisible(false);
                    hwFreeInfoButton.setVisible(false);
                    hwFreeInfoButton.setManaged(false);
                }
            }

            Integer countIpv4Avail = counts.getCount(ServerGlimpse.AdvertisementInfo.CountedObject.IPV4, ServerGlimpse.AdvertisementInfo.CountScope.AVAILABLE);
            Integer countIpv4Total = counts.getCount(ServerGlimpse.AdvertisementInfo.CountedObject.IPV4, ServerGlimpse.AdvertisementInfo.CountScope.ALL);

            if (countIpv4Total != null && countIpv4Total > 0) {
                ipAvailabiltyMeter.setAvailable(countIpv4Avail);
                ipAvailabiltyMeter.setTotal(countIpv4Total);
                ipAvailabiltyMeter.setUndetermined(false);
                ipFreeInfoLabel.setText(countIpv4Avail + " of " + countIpv4Total);
            } else {
                ipAvailabiltyMeter.setAvailable(0);
                ipAvailabiltyMeter.setTotal(0);
                ipAvailabiltyMeter.setUndetermined(true);
                ipFreeInfoLabel.setText("-");
            }

            String countsTime;
            if (counts.getCountDate() != null) {
                countsTime = TimeUtil.calcAgo(counts.getCountDate());
            } else {
                countsTime = "at unknown time";
            }

            updateCountsDateLabel.setText("Counts updated " + countsTime + ".");
        } else {
            ipAvailabiltyMeter.setAvailable(0);
            ipAvailabiltyMeter.setTotal(0);
            ipAvailabiltyMeter.setUndetermined(true);
            ipFreeInfoLabel.setText("-");

            freeLabel.setText("Free resources:");
            availabiltyMeter.setAvailable(0);
            availabiltyMeter.setTotal(0);
            availabiltyMeter.setUndetermined(true);
            freeInfoLabel.setText("-");
            freeHwInfoLabel.setVisible(false);
            hwFreeInfoButton.setVisible(false);
            hwFreeInfoButton.setManaged(false);

            updateCountsDateLabel.setText("Count info not available.");
        }
    }

    public void showMaintenance(@Nullable Collection<? extends MaintenanceInfo> maintenance) {
//        if (maintenance == null) {
////        //no maintenance:
//            maintenanceInfoButton.setVisible(false);
//            maintenanceLabel.setVisible(false);
//            maintenanceGlyph.setVisible(false);
//            return;
//        }
//        maintenanceInfoButton.setVisible(true);
//        maintenanceLabel.setVisible(true);
//        maintenanceGlyph.setVisible(true);

        Optional<? extends MaintenanceInfo> activeMaintenance;
        Optional<? extends MaintenanceInfo> upcomingMaintenance;
        if (maintenance != null && !maintenance.isEmpty()) {
            activeMaintenance =
                    maintenance.stream()
                            .filter(mm -> mm.getStartTime() == null || mm.getStartTime().isBefore(ZonedDateTime.now()))
                            .filter(mm -> mm.getEndTime() == null || mm.getEndTime().isAfter(ZonedDateTime.now()))
                            .sorted(MAINTENANCE_MESSAGE_COMPARATOR)
                            .findFirst();

            upcomingMaintenance =
                    maintenance.stream()
                            .filter(mm -> mm.getStartTime() != null && mm.getStartTime().isAfter(ZonedDateTime.now()))
                            .sorted(MAINTENANCE_MESSAGE_COMPARATOR)
                            .findFirst();
        } else {
            activeMaintenance = Optional.empty();
            upcomingMaintenance = Optional.empty();
        }
        if (activeMaintenance.isPresent()) {
            StringBuilder maintenanceMsg = new StringBuilder("Currently in maintenance ");

            ZonedDateTime startTime = activeMaintenance.get().getStartTime();
            if (startTime != null)
                maintenanceMsg.append("from ")
                        .append(DateTimeUtils.DATE_HOUR_MINUTES_FORMATTER.format(
                                startTime.withZoneSameInstant(ZoneId.systemDefault()))
                        )
                        .append(" ");
            ZonedDateTime endTime = activeMaintenance.get().getEndTime();
            if (endTime != null)
                maintenanceMsg.append("until ")
                        .append(DateTimeUtils.DATE_HOUR_MINUTES_FORMATTER.format(
                                endTime.withZoneSameInstant(ZoneId.systemDefault()))
                        );

            maintenanceLabel.setText(maintenanceMsg.toString());
            maintenanceGlyph.setTextFill(Color.RED);
        } else if (upcomingMaintenance.isPresent()) {
            StringBuilder maintenanceMsg = new StringBuilder("Upcoming maintenance ");

            ZonedDateTime startTime = upcomingMaintenance.get().getStartTime();
            if (startTime != null)
                maintenanceMsg.append("from ")
                        .append(DateTimeUtils.DATE_HOUR_MINUTES_FORMATTER.format(
                                startTime.withZoneSameInstant(ZoneId.systemDefault()))
                        )
                        .append(" ");
            ZonedDateTime endTime = upcomingMaintenance.get().getEndTime();
            if (endTime != null)
                maintenanceMsg.append("until ")
                        .append(DateTimeUtils.DATE_HOUR_MINUTES_FORMATTER.format(
                                endTime.withZoneSameInstant(ZoneId.systemDefault()))
                        );

            maintenanceLabel.setText(maintenanceMsg.toString());
            maintenanceGlyph.setTextFill(Color.ORANGE);
        } else {
            maintenanceLabel.setText("None planned");
            maintenanceGlyph.setTextFill(Color.GRAY);
        }

        maintenanceInfoButton.setDisable(!activeMaintenance.isPresent() && !upcomingMaintenance.isPresent());
    }

    @FXML
    private void refresh() {
        //refresh is not that usefull, as the time shown is that ONE THE SERVER.
        //   therefor, a user will be confused after a refresh...
        if (config != null) {
            LOG.debug("Updating ServerGlimpses");
            config.forceRefreshServerGlimpses();
//            updateDateLabel.setText(updateDateLabel.getText()+"...");
        } else {
            LOG.warn("Internal error: Cannot refresh");
//            updateDateLabel.setText("(Internal error: Cannot refresh)");
        }
    }

    @FXML
    private void toggleHwFree() {
//        showHwFree = !showHwFree;

        AdvertisementCounts counts = config.getServerNodeCounts(server).get();
        Integer countAvail;
        Integer countTotal;
        String typename;
        switch (type.get()) {
            case RAW_PC: {
                countAvail = counts.getCount(ServerGlimpse.AdvertisementInfo.CountedObject.RAW_PC, ServerGlimpse.AdvertisementInfo.CountScope.AVAILABLE);
                countTotal = counts.getCount(ServerGlimpse.AdvertisementInfo.CountedObject.RAW_PC, ServerGlimpse.AdvertisementInfo.CountScope.ALL);
                typename = "Bare Metal";
                break;
            }
            case VM: {
                countAvail = counts.getCount(ServerGlimpse.AdvertisementInfo.CountedObject.VM, ServerGlimpse.AdvertisementInfo.CountScope.AVAILABLE);
                countTotal = counts.getCount(ServerGlimpse.AdvertisementInfo.CountedObject.VM, ServerGlimpse.AdvertisementInfo.CountScope.ALL);
                typename = "VM";
                break;
            }
            default: {
                countAvail = counts.getCount(ServerGlimpse.AdvertisementInfo.CountedObject.ANY, ServerGlimpse.AdvertisementInfo.CountScope.AVAILABLE);
                countTotal = counts.getCount(ServerGlimpse.AdvertisementInfo.CountedObject.ANY, ServerGlimpse.AdvertisementInfo.CountScope.ALL);
                typename = "Any";
                break;
            }
        }

        GridPane gridPane = new GridPane();
        gridPane.getStyleClass().add("grid");
        int rowIndex = 0;
//        gridPane.setPadding(new Insets(10.0));
        gridPane.setHgap(10.0);
        gridPane.setVgap(2.0);
        ColumnConstraints colA = new ColumnConstraints();
        colA.setHgrow(Priority.ALWAYS);
        ColumnConstraints colB = new ColumnConstraints();
        colB.setHgrow(Priority.ALWAYS);
        ColumnConstraints colC = new ColumnConstraints();
        colC.setHgrow(Priority.ALWAYS);
        gridPane.getColumnConstraints().addAll(colA, colB, colC);

        Label headerA = new Label("Type");
        headerA.setStyle("-fx-font-weight: bold");
        Label headerB = new Label("Available");
        headerB.setStyle("-fx-font-weight: bold");
        Label headerC = new Label("Total");
        headerC.setStyle("-fx-font-weight: bold");
        gridPane.getChildren().addAll(headerA, headerB, headerC);
        GridPane.setRowIndex(headerA, rowIndex);
        GridPane.setColumnIndex(headerA, 0);
        GridPane.setRowIndex(headerB, rowIndex);
        GridPane.setColumnIndex(headerB, 1);
        GridPane.setRowIndex(headerC, rowIndex);
        GridPane.setColumnIndex(headerC, 2);
        headerA.getStyleClass().add("cell");
        headerB.getStyleClass().add("cell");
        headerC.getStyleClass().add("cell");
        headerA.getStyleClass().add("header");
        headerB.getStyleClass().add("header");
        headerC.getStyleClass().add("header");
        headerA.setMaxWidth(10000.0);
        headerB.setMaxWidth(10000.0);
        headerC.setMaxWidth(10000.0);

//        rowIndex++;
//        Label countAllA = new Label(typename); countAllA.setStyle("-fx-font-weight: bold");
//        Label countAllB = new Label(""+countAvail);
//        Label countAllC = new Label(""+countTotal);
//        popoverRoot.getChildren().addAll(countAllA, countAllB, countAllC);
//        GridPane.setRowIndex(countAllA, rowIndex); GridPane.setColumnIndex(countAllA, 0);
//        GridPane.setRowIndex(countAllB, rowIndex); GridPane.setColumnIndex(countAllB, 1);
//        GridPane.setRowIndex(countAllC, rowIndex); GridPane.setColumnIndex(countAllC, 2);

        rowIndex++;
        rowIndex++;

//        popoverRoot.getChildren().add(new Label(countAvail + " of " + countTotal));
        List<String> hwCountKeys = new ArrayList<>(counts.getHwCountKeys());
        Collections.sort(hwCountKeys, Comparator.comparing(s -> s.toLowerCase().replaceAll("[^a-z0-9]*", "")));
        LOG.debug("Health is showing hw type counts: " + hwCountKeys);
        for (String hwType : hwCountKeys) {
            Integer allCount = counts.getHwCount(hwType, AdvertisementCounts.CountScope.ALL);
            Integer availCount = counts.getHwCount(hwType, AdvertisementCounts.CountScope.AVAILABLE);
//            popoverRoot.getChildren().add(new Label(hwType + ": " + availCount + " of " + allCount ));

            rowIndex++;
            Label countA = new Label(hwType);
            Label countB = new Label("" + availCount);
            Label countC = new Label("" + allCount);
            gridPane.getChildren().addAll(countA, countB, countC);
            GridPane.setRowIndex(countA, rowIndex);
            GridPane.setColumnIndex(countA, 0);
            GridPane.setRowIndex(countB, rowIndex);
            GridPane.setColumnIndex(countB, 1);
            GridPane.setRowIndex(countC, rowIndex);
            GridPane.setColumnIndex(countC, 2);
            countA.getStyleClass().add("cell");
            countB.getStyleClass().add("cell");
            countC.getStyleClass().add("cell");
            countA.setMaxWidth(10000.0);
            countB.setMaxWidth(10000.0);
            countC.setMaxWidth(10000.0);
        }

        StackPane popOverRoot = new StackPane();
        popOverRoot.getChildren().add(gridPane);
        popOverRoot.setPadding(new Insets(10));
        PopOver popOver = new PopOver(popOverRoot);

        popOver.show(hwFreeInfoButton);

        //showCounts(config.getServerNodeCounts(server).get());
    }

    @FXML
    private void showMaintenanceInfo() {
        ObservableList<MaintenanceInfo> maintenances = config.getServerMaintenance(server);
        assert maintenances != null;
        assert !maintenances.isEmpty();

        List<MaintenanceInfo> sortedMaintenances = new ArrayList<>(maintenances);
        sortedMaintenances.sort(MAINTENANCE_MESSAGE_COMPARATOR);


        Alert infoAlert = new Alert(Alert.AlertType.INFORMATION);
        infoAlert.setHeaderText("Planned maintenance");
        infoAlert.setContentText(formatMaintenanceInfos(sortedMaintenances));
        infoAlert.setResizable(true);
        infoAlert.showAndWait();
    }

    private static void setGlyphColor(@Nonnull Glyph glyph, @Nullable Integer value) {
        if (value == null) {
            glyph.setTextFill(Color.GRAY);
        } else if (value > 95) {
            glyph.setTextFill(Color.GREEN);
        } else if (value > 66) {
            glyph.setTextFill(Color.YELLOWGREEN);
        } else if (value > 33) {
            glyph.setTextFill(Color.ORANGE);
        } else {
            glyph.setTextFill(Color.RED);
        }
    }

    public JFedGuiConfig.ResourceType getType() {
        return type.get();
    }

    public ObjectProperty<JFedGuiConfig.ResourceType> typeProperty() {
        return type;
    }

    public void setType(JFedGuiConfig.ResourceType type) {
        this.type.set(type);
    }


    @Nonnull
    public static String formatMaintenanceInfos(List<MaintenanceInfo> maintenanceInfos) {
        StringBuilder maintenanceMsg = new StringBuilder();

        for (MaintenanceInfo activeMaintenance : maintenanceInfos) {
            if (activeMaintenance.getStartTime() != null && activeMaintenance.getEndTime() != null) {
                maintenanceMsg.append("From ")
                        .append(DateTimeUtils.DATE_HOUR_MINUTES_FORMATTER.format(activeMaintenance.getStartTime().withZoneSameInstant(ZoneId.systemDefault())))
                        .append(" until ")
                        .append(DateTimeUtils.DATE_HOUR_MINUTES_FORMATTER.format(activeMaintenance.getEndTime().withZoneSameInstant(ZoneId.systemDefault())))
                        .append(": ");
            } else if (activeMaintenance.getStartTime() != null) {
                maintenanceMsg.append("From ")
                        .append(DateTimeUtils.DATE_HOUR_MINUTES_FORMATTER.format(activeMaintenance.getStartTime().withZoneSameInstant(ZoneId.systemDefault())))
                        .append(": ");
            } else if (activeMaintenance.getEndTime() != null) {
                maintenanceMsg
                        .append("Until ")
                        .append(DateTimeUtils.DATE_HOUR_MINUTES_FORMATTER.format(activeMaintenance.getEndTime().withZoneSameInstant(ZoneId.systemDefault())))
                        .append(": ");
            }

            maintenanceMsg.append(activeMaintenance.getMessage()).append("\n\n");
        }
        return maintenanceMsg.toString().trim();
    }
}
